import javax.swing.*;

import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

public class ArithmeticOperatorView extends JFrame implements ActionListener
{
   private Random rng;
   private int xInit;
   private String expression;
   
   private boolean answerSubmitted;
   
   private JTextArea expressionTextArea;
   private JTextField answerTextField;
   private JButton answerButton;
   
   static public String SUBMIT_ACTION = "SUBMIT";
   
   public ArithmeticOperatorView()
   {
      Date now = new Date();
      this.rng = new Random(now.getTime());
      this.xInit = 0;
      this.expression = "";
      this.answerSubmitted = false;
      
      Font font = new Font(Font.MONOSPACED, Font.PLAIN, 16);
      
      this.expressionTextArea = new JTextArea("hi", 3, 40);
      this.expressionTextArea.setEditable(false);
      this.expressionTextArea.setFont(font);
      
      JLabel answerLabel = new JLabel("What is the value of y?");
      
      this.answerTextField = new JTextField();
      this.answerTextField.setFont(font);
      
      this.answerButton = new JButton("Submit");
      this.answerButton.addActionListener(this);
      this.answerButton.setActionCommand(ArithmeticOperatorView.SUBMIT_ACTION);
      
      GroupLayout layout = new GroupLayout(this.getContentPane());
      this.getContentPane().setLayout(layout);
      layout.setHorizontalGroup(layout.createSequentialGroup()
         .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addComponent(this.expressionTextArea)
            .addGroup(layout.createSequentialGroup()
               .addComponent(answerLabel)
               .addComponent(this.answerTextField)
               .addComponent(answerButton))));
      layout.setVerticalGroup(layout.createSequentialGroup()
         .addComponent(this.expressionTextArea)
         .addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
            .addComponent(answerLabel)
            .addComponent(this.answerTextField)
            .addComponent(answerButton)));
      
      this.pack();
      this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }
   
   @Override
   public void actionPerformed(ActionEvent event)
   {
      if (event.getActionCommand().equals(ArithmeticOperatorView.SUBMIT_ACTION))
      {
         this.answerSubmitted = true;
      }
   }
   
   public boolean answerSubmitted()
   {
      return this.answerSubmitted;
   }
   
   public String getSubmittedAnswer()
   {
      return this.answerTextField.getText().trim();
   }
   
   public void showCongratulation()
   {
      JOptionPane.showMessageDialog(this,
         "You got the correct answer.",
         "Congratulations!",
         JOptionPane.PLAIN_MESSAGE);
   }
   
   public void showSolution(String sol)
   {
      //this.expressionTextArea.append(sol);
      JOptionPane.showMessageDialog(this,
         sol,
         "Solution",
         JOptionPane.ERROR_MESSAGE);
      
   }
   
   public String makeExpression(int numOperands, int minOperandValue, int maxOperandValue)
   {
      if (numOperands < 1)
      {
         numOperands = 1;
      }
      if (minOperandValue >= maxOperandValue)
      {
         minOperandValue = maxOperandValue - 1;
      }
      List<String> operands = new ArrayList<String>();
      for(int i = 0; i < numOperands; ++i)
      {
         operands.add(String.valueOf(
            this.getRandomInt(minOperandValue, maxOperandValue)));
      }
      final int xIndex = this.rng.nextInt(numOperands);
      operands.set(xIndex, this.getRandomXOperand());
      StringBuffer buf = new StringBuffer();
      buf.append(operands.get(0));
      for(int i = 1; i < numOperands; ++i)
      {
         buf.append(' ');
         buf.append(this.getRandomBinaryOperator());
         buf.append(' ');
         buf.append(operands.get(i));
      }
      this.expression = buf.toString();
      this.xInit = this.getRandomInt(minOperandValue, maxOperandValue);
      this.expressionTextArea.setText("int x = " + xInit + ";\n");
      this.expressionTextArea.append("int y = " + this.expression + ";\n");
      this.answerSubmitted = false;
      this.answerTextField.setText("");
      return this.expression;
   }
   
   public String getExpression()
   {
      return this.expression;
   }
   
   public int getX()
   {
      return this.xInit;
   }
   
   private int getRandomInt(int minOperandValue, int maxOperandValue)
   {
      final int RANGE = maxOperandValue - minOperandValue + 1;
      return this.rng.nextInt(RANGE) + minOperandValue;
   }
   
   private char getRandomBinaryOperator()
   {
      final int PLUS = 0;
      final int MINUS = 1;
      final int MULTIPLY = 2;
      final int DIVIDE = 3;
      final int REMAINDER = 4;
      final int NUM_OPERATOR_TYPES = 5;
      char op = 0;
      switch(this.rng.nextInt(NUM_OPERATOR_TYPES))
      {
      case PLUS:
         op = '+';
         break;
      case MINUS:
         op = '-';
         break;
      case MULTIPLY:
         op = '*';
         break;
      case DIVIDE:
         op = '/';
         break;
      case REMAINDER:
         op = '%';
         break;
      }
      return op;
   }
   
   private String getRandomXOperand()
   {
      final int PLUS = 0;
      final int MINUS = 1;
      final int PRE_PLUSPLUS = 2;
      final int PRE_MINUSMINUS = 3;
      final int POST_PLUSPLUS = 4;
      final int POST_MINUSMINUS = 5;
      final int OP = this.rng.nextInt(10);
      String x = "";
      switch (OP)
      {
      case PLUS:
         x = "+x";
         break;
      case MINUS:
         x = "-x";
         break;
      case PRE_PLUSPLUS:
         x = "++x";
         break;
      case PRE_MINUSMINUS:
         x = "--x";
         break;
      case POST_PLUSPLUS:
         x = "x++";
         break;
      case POST_MINUSMINUS:
         x = "x--";
         break;
      default:
         x = "x";
         break;
      }
      return x;
   }
}
